@@ -3,6 +3,10 @@ from ML_management.mlmanagement import mlmanagement
 set_experiment = mlmanagement.set_experiment
 start_run = mlmanagement.start_run
 log_model = mlmanagement.log_model
+log_object_src = mlmanagement.log_object_src
+log_executor_src = mlmanagement.log_executor_src
+log_dataset_loader_src = mlmanagement.log_dataset_loader_src
+log_model_src = mlmanagement.log_model_src
 log_metric = mlmanagement.log_metric
 set_tag = mlmanagement.set_tag
 autolog = mlmanagement.autolog
@@ -15,5 +19,9 @@ log_param = mlmanagement.log_param
 log_params = mlmanagement.log_params
 set_server_url = mlmanagement.set_server_url
 get_server_url = mlmanagement.get_server_url
+set_minio_url = mlmanagement.set_minio_url
+set_minio_credentials = mlmanagement.set_minio_credentials
+get_minio_url = mlmanagement.get_minio_url
+set_mlm_credentials = mlmanagement.set_mlm_credentials
 get_server_websocket_url = mlmanagement.get_server_websocket_url
 MlflowClient = mlmanagement.MlflowClient
