@@ -2,8 +2,8 @@
 from typing import List
 
 
-class DictKeysMustBeStrings(Exception):
-    """Define DictKeysMustBeStrings exception."""
+class DictKeysMustBeStringsError(Exception):
+    """Define DictKeysMustBeStringsError exception."""
 
     def __init__(self, annotation: str):
         self.annotation = annotation
@@ -13,11 +13,11 @@ class DictKeysMustBeStrings(Exception):
 
     def __reduce__(self):
         """Define reduce method to make exception picklable."""
-        return (DictKeysMustBeStrings, (self.annotation))
+        return (DictKeysMustBeStringsError, (self.annotation))
 
 
-class InvalidSchema(Exception):
-    """Define InvalidSchema exception."""
+class InvalidSchemaError(Exception):
+    """Define InvalidSchemaError exception."""
 
     def __init__(self, schema: dict, original_message: str):
         self.schema = schema
@@ -31,11 +31,11 @@ class InvalidSchema(Exception):
 
     def __reduce__(self):
         """Define reduce method to make exception picklable."""
-        return (InvalidSchema, (self.schema, self.original_message))
+        return (InvalidSchemaError, (self.schema, self.original_message))
 
 
-class NoAnnotation(Exception):
-    """Define NoAnnotation exception."""
+class NoAnnotationError(Exception):
+    """Define NoAnnotationError exception."""
 
     def __init__(self, arg: str):
         self.arg = arg
@@ -45,11 +45,11 @@ class NoAnnotation(Exception):
 
     def __reduce__(self):
         """Define reduce method to make exception picklable."""
-        return (NoAnnotation, (self.arg))
+        return (NoAnnotationError, (self.arg))
 
 
-class UnsupportedType(Exception):
-    """Define UnsupportedType exception."""
+class UnsupportedTypeError(Exception):
+    """Define UnsupportedTypeError exception."""
 
     def __init__(self, annotation: str, supported_types: List[str]):
         self.annotation = annotation
@@ -63,11 +63,11 @@ class UnsupportedType(Exception):
 
     def __reduce__(self):
         """Define reduce method to make exception picklable."""
-        return (UnsupportedType, (self.annotation, self.supported_types))
+        return (UnsupportedTypeError, (self.annotation, self.supported_types))
 
 
-class FunctionContainsVarArgs(Exception):
-    """Define FunctionContainsVarArgs exception."""
+class FunctionContainsVarArgsError(Exception):
+    """Define FunctionContainsVarArgsError exception."""
 
     def __init__(self, function_name: str):
         self.function_name = function_name
@@ -79,11 +79,11 @@ class FunctionContainsVarArgs(Exception):
 
     def __reduce__(self):
         """Define reduce method to make exception picklable."""
-        return (FunctionContainsVarArgs, (self.function_name))
+        return (FunctionContainsVarArgsError, (self.function_name))
 
 
-class FunctionContainsVarKwArgs(Exception):
-    """Define FunctionContainsVarKwArgs exception."""
+class FunctionContainsVarKwArgsError(Exception):
+    """Define FunctionContainsVarKwArgsError exception."""
 
     def __init__(self, function_name: str):
         self.function_name = function_name
@@ -95,4 +95,4 @@ class FunctionContainsVarKwArgs(Exception):
 
     def __reduce__(self):
         """Define reduce method to make exception picklable."""
-        return (FunctionContainsVarKwArgs, (self.function_name))
+        return (FunctionContainsVarKwArgsError, (self.function_name))
