@@ -5,6 +5,7 @@ ML_manager sends request to our server on /mlflow endpoint
 Server calls original mlflow function accordingly
 """
 import atexit
+import importlib
 import inspect
 import io
 import os
@@ -13,6 +14,7 @@ import sys
 import tempfile
 import traceback
 import zipfile
+from pathlib import Path
 from typing import Any, Dict, List, Optional, Union
 
 import cloudpickle
@@ -21,7 +23,7 @@ import pandas
 from scipy.sparse import csc_matrix, csr_matrix
 
 import mlflow
-from ML_management.executor_template.upload_model_mode import UploadModelMode
+from ML_management.executor.upload_model_mode import UploadModelMode
 from ML_management.mlmanagement import variables
 from ML_management.mlmanagement.abstract_mlflow_client import AbstractMlflowClient
 from ML_management.mlmanagement.base_exceptions import PylintError
@@ -29,8 +31,13 @@ from ML_management.mlmanagement.mlmanager import request_for_function
 from ML_management.mlmanagement.model_type import ModelType
 from ML_management.mlmanagement.module_finder import ModuleFinder
 from ML_management.mlmanagement.server_mlmanager_exceptions import ModelTypeIsNotFoundError
-from ML_management.mlmanagement.utils import _load_model_type
-from ML_management.mlmanagement.variables import active_run_stack
+from ML_management.mlmanagement.utils import _load_model_type, is_model_name_valid, validate_predict_config
+from ML_management.mlmanagement.variables import (
+    EXPERIMENT_NAME_FOR_DATASET_LOADER,
+    EXPERIMENT_NAME_FOR_EXECUTOR,
+    FILENAME_FOR_INFERENCE_CONFIG,
+    active_run_stack,
+)
 from ML_management.mlmanagement.visibility_options import VisibilityOptions
 from mlflow import ActiveRun
 from mlflow.entities import Experiment, Run, RunStatus
@@ -118,6 +125,8 @@ def _add_job_registration_metainfo(
     dataset_loader_version,
     role_model_map,
     new_model_names,
+    prepare_new_models_inference,
+    descriptions,
     experiment_name,
     periodic_type,
     cron_expression,
@@ -126,6 +135,7 @@ def _add_job_registration_metainfo(
     data_params_json,
     collector_name,
     gpu,
+    additional_system_packages,
 ):
     """Add job metainfo on job registration. For internal use only."""
     return request_for_function(inspect.currentframe())
@@ -157,11 +167,12 @@ def _add_job_fail_metainfo(
 
 def log_model(
     artifact_path,
-    description: Optional[str] = None,
+    description: Optional[str],
+    model_version_tags: Optional[Dict[str, str]] = None,
     code_path=None,
     conda_env=None,
     python_model=None,
-    artifacts=None,
+    artifacts: Optional[dict] = None,
     registered_model_name: str = "default_name",
     signature: mlflow.models.signature.ModelSignature = None,
     input_example: Union[
@@ -183,17 +194,33 @@ def log_model(
     root_module_name: str = "__main__",
     linter_check: bool = True,
     start_build: bool = True,
+    create_venv_pack: bool = False,
 ):
     """
     Log a Pyfunc model with custom inference logic and optional data dependencies as an MLflow artifact.
 
     Current run is using.
+    Parameter registered_model_name must be not empty string,
+        consist of alphanumeric characters, '_'
+        and must start and end with an alphanumeric character.
+        Validation regexp: "(([A-Za-z0-9][A-Za-z0-9_]*)?[A-Za-z0-9])+"
     You cannot specify the parameters: loader_module, data_path and the parameters: python_model, artifacts together.
     """
     if extra_modules_names and used_modules_names:
         raise RuntimeError("Parameters 'extra_modules_names' and 'used_modules_names' cannot be used at the same time.")
     if upload_model_mode == UploadModelMode.none:
         raise RuntimeError("You can't log a model using the 'upload_model_mode' parameter set to none.")
+    if not is_model_name_valid(registered_model_name):
+        raise RuntimeError(
+            "Parameter 'registered_model_name' must be not empty string, "
+            "consist of alphanumeric characters, '_' "
+            "and must start and end with an alphanumeric character."
+            "Validation regexp: '(([A-Za-z0-9][A-Za-z0-9_]*)?[A-Za-z0-9])+'"
+        )
+    if create_venv_pack:
+        if not isinstance(artifacts, dict):
+            raise RuntimeError("You must provide config for model serving in artifacts, if create_venv_pack=True")
+        validate_predict_config(path=artifacts.get(FILENAME_FOR_INFERENCE_CONFIG))
     start_run_if_not_exist()
     if used_modules_names is not None:
         submodules = set(used_modules_names)
@@ -214,7 +241,7 @@ def log_model(
                             "--clear-cache-post-run=True",
                             "--disable=all",
                             "--enable=E",
-                            "--disable=E0401, E0110, E1101",
+                            "--disable=E0401, E1101",
                             "--fail-on=E",
                             "--reports=n",
                             "--score=n",
@@ -241,6 +268,152 @@ def log_model(
             cloudpickle.unregister_pickle_by_value(sys.modules[module_name])
 
 
+def log_object_src(
+    artifact_path,
+    model_path: str,
+    description: str,
+    model_version_tags: Optional[Dict[str, str]] = None,
+    code_path=None,
+    conda_env=None,
+    python_model=None,
+    registered_model_name: str = "default_name",
+    signature: mlflow.models.signature.ModelSignature = None,
+    input_example: Union[
+        pandas.core.frame.DataFrame, numpy.ndarray, dict, list, csr_matrix, csc_matrix, str, bytes
+    ] = None,
+    await_registration_for: int = 300,
+    pip_requirements=None,
+    extra_pip_requirements=None,
+    metadata=None,
+    source_model_name=None,
+    source_model_version=None,
+    source_executor_name=None,
+    source_executor_version=None,
+    source_executor_role=None,
+    upload_model_mode=None,
+    visibility=None,
+    start_build: bool = True,
+    create_venv_pack: bool = False,
+):
+    """
+    Log a Pyfunc model with custom inference logic and optional data dependencies as an MLflow artifact.
+
+    Current run is using.
+    Parameter registered_model_name must be not empty string,
+            consist of alphanumeric characters, '_'
+            and must start and end with an alphanumeric character.
+            Validation regexp: "(([A-Za-z0-9][A-Za-z0-9_]*)?[A-Za-z0-9])+"
+    You cannot specify the parameters: loader_module, data_path and the parameters: python_model, artifacts together.
+    """
+    if not os.path.exists(os.path.join(model_path, "conda.yaml")):
+        raise Exception("There is no conda.yaml file.")
+
+    if upload_model_mode == UploadModelMode.none:
+        raise RuntimeError("You can't log a model using the 'upload_model_mode' parameter set to none.")
+    if not is_model_name_valid(registered_model_name):
+        raise RuntimeError(
+            "Parameter 'registered_model_name' must be not empty string, "
+            "consist of alphanumeric characters, '_'"
+            "and must start and end with an alphanumeric character."
+            "Validation regexp: '(([A-Za-z0-9][A-Za-z0-9_]*)?[A-Za-z0-9])+'"
+        )
+    if create_venv_pack:
+        validate_predict_config(path=os.path.join(f"{model_path}", "artifacts", f"{FILENAME_FOR_INFERENCE_CONFIG}"))
+    start_run_if_not_exist()
+    python_path = sys.path
+    try:
+        parts = Path(model_path).parts
+
+        if os.path.isabs(model_path):
+            sys.path.append(str(Path(*parts[:-1])))
+
+            python_model = importlib.import_module(parts[-1]).get_object()  # noqa: F841
+        else:
+            python_model = importlib.import_module(".".join(parts)).get_object()  # noqa: F841
+
+        result = request_for_function(inspect.currentframe(), ["pyfunc"])
+    except Exception as err:
+        raise err
+    finally:
+        sys.path = python_path
+
+    return result
+
+
+def log_executor_src(
+    model_path: str,
+    registered_name: str,
+    description: str,
+    artifact_path: str = "",
+    start_build: bool = False,
+    visibility: VisibilityOptions = VisibilityOptions.PRIVATE,
+):
+    old_experiment_name = variables.active_experiment_name
+
+    try:
+        set_experiment(EXPERIMENT_NAME_FOR_EXECUTOR, visibility=VisibilityOptions.PUBLIC)
+
+        with start_run(nested=True):
+            log_object_src(
+                artifact_path=artifact_path,
+                description=description,
+                registered_model_name=registered_name,
+                start_build=start_build,
+                model_path=model_path,
+                visibility=visibility,
+            )
+
+    except Exception as err:
+        raise err
+    finally:
+        variables.active_experiment_name = old_experiment_name
+
+
+def log_dataset_loader_src(
+    model_path: str,
+    registered_name: str,
+    description: str,
+    artifact_path: str = "",
+):
+    old_experiment_name = variables.active_experiment_name
+    try:
+        set_experiment(EXPERIMENT_NAME_FOR_DATASET_LOADER, visibility=VisibilityOptions.PUBLIC)
+        with start_run(nested=True):
+            log_object_src(
+                artifact_path=artifact_path,
+                description=description,
+                registered_model_name=registered_name,
+                model_path=model_path,
+            )
+    except Exception as err:
+        raise err
+    finally:
+        variables.active_experiment_name = old_experiment_name
+
+
+def log_model_src(
+    model_path: str,
+    registered_name: str,
+    description: str,
+    artifact_path: str = "",
+    start_build: bool = True,
+    model_version_tags: Optional[Dict[str, str]] = None,
+    create_venv_pack: bool = False,
+):
+    """Parameter registered_name should match regexp "(([A-Za-z0-9][A-Za-z0-9_]*)?[A-Za-z0-9])+"."""
+    with start_run(nested=True):
+        log_object_src(
+            artifact_path=artifact_path,
+            description=description,
+            registered_model_name=registered_name,
+            model_path=model_path,
+            visibility=VisibilityOptions.PRIVATE,
+            start_build=start_build,
+            model_version_tags=model_version_tags,
+            create_venv_pack=create_venv_pack,
+        )
+
+
 def log_metric(key: str, value: float, step: Optional[int] = None) -> None:
     """
     Log a metric under the current run. If no run is active, this method will create a new active run.
@@ -363,21 +536,59 @@ def set_server_url(url: str) -> None:
     """
     Set server URL.
 
-    If you set the URL using this function, it takes precedence over the URL from the environment variable 'server_url'.
+    If you set the URL using this function,
+    it takes precedence over the URL from the environment variable 'server_url'.
     """
     variables.server_url = url
 
 
+def set_minio_url(url: str) -> None:
+    """
+    Set minio URL.
+
+    If you set the URL using this function,
+    it takes precedence over the URL from the environment variable 'minio_url'.
+    """
+    variables.minio_url = url
+
+
+def set_minio_credentials(login: str, password: str) -> None:
+    """
+    Set login and password for minio.
+
+    If you set the credentials using this function,
+    it takes precedence over the credentials from the environment variables.
+    """
+    variables.minio_username = login
+    variables.minio_password = password
+
+
+def set_mlm_credentials(login: str, password: str) -> None:
+    """
+    Set login and password for mlmanagement.
+
+    If you set the credentials using this function,
+    it takes precedence over the credentials from the environment variables.
+    """
+    variables.mlm_login = login
+    variables.mlm_password = password
+
+
 def get_server_url() -> str:
     """Get the current server URL."""
     return variables._get_server_url()
 
 
+def get_minio_url() -> str:
+    """Get the current minio URL."""
+    return variables._get_minio_url()
+
+
 def get_server_websocket_url() -> str:
     """Get the current websocket server URL."""
     url = get_server_url()
     splitted_url = url.split("/")
-    splitted_url[0] = "wss:"
+    splitted_url[0] = "wss:" if "https" in url else "ws:"
     return "/".join(splitted_url)
 
 
@@ -400,28 +611,34 @@ def set_model_version_requirements(name: str, version: int, model_type: ModelTyp
         print(f"Requirements for {name} weren't installed correctly.")
 
 
-def load_model(name: str, version: int, install_requirements: bool = False, unwrap: bool = True):
+def load_model(
+    name: str, version: int, install_requirements: bool = False, unwrap: bool = True, dst_path: Optional[str] = None
+):
     """Download all model's files for loading model locally."""
     if install_requirements:
         set_model_version_requirements(name=name, version=version, model_type=ModelType.MODEL)
     run_id = MlflowClient().get_model_version(name, version).run_id
-    return _load_model_type(run_id, unwrap)
+    return _load_model_type(run_id, unwrap, dst_path)
 
 
-def load_dataset(name: str, version: int, install_requirements: bool = False, unwrap: bool = True):
+def load_dataset(
+    name: str, version: int, install_requirements: bool = False, unwrap: bool = True, dst_path: Optional[str] = None
+):
     """Download all model's files for loading model locally."""
     if install_requirements:
         set_model_version_requirements(name=name, version=version, model_type=ModelType.DATASET_LOADER)
     run_id = MlflowClient().get_dataset_loader_version(name, version).run_id
-    return _load_model_type(run_id, unwrap)
+    return _load_model_type(run_id, unwrap, dst_path)
 
 
-def load_executor(name: str, version: int, install_requirements: bool = False, unwrap: bool = True):
+def load_executor(
+    name: str, version: int, install_requirements: bool = False, unwrap: bool = True, dst_path: Optional[str] = None
+):
     """Download all model's files for loading model locally."""
     if install_requirements:
         set_model_version_requirements(name=name, version=version, model_type=ModelType.EXECUTOR)
     run_id = MlflowClient().get_executor_version(name, version).run_id
-    return _load_model_type(run_id, unwrap)
+    return _load_model_type(run_id, unwrap, dst_path)
 
 
 atexit.register(end_run)
