@@ -9,11 +9,13 @@ import cloudpickle
 from requests_toolbelt import MultipartEncoder
 
 import mlflow
+from ML_management.loader.loader import CONFIG_KEY_ARTIFACTS
 from ML_management.mlmanagement import variables
 from ML_management.mlmanagement.base_exceptions import MLMServerError
 from ML_management.mlmanagement.jsonschema_inference import infer_jsonschema
 from ML_management.mlmanagement.model_type import ModelType
 from ML_management.mlmanagement.server_mlmanager_exceptions import (
+    AuthError,
     InvalidExperimentNameError,
     InvalidVisibilityOptionError,
     ModelTypeIsNotFoundError,
@@ -32,18 +34,18 @@ def create_kwargs(frame, is_it_class_function=False):
     """Get name and kwargs of function by its frame."""
     function_name = inspect.getframeinfo(frame)[2]  # get name of function
     _, _, _, kwargs = inspect.getargvalues(frame)  # get kwargs of function
-    if "self" in kwargs:
-        del kwargs["self"]
+    kwargs.pop("self", None)
+    kwargs.pop("parts", None)
+    kwargs.pop("python_path", None)
     if is_it_class_function:
-        if "dst_path" in kwargs:
-            del kwargs["dst_path"]
+        kwargs.pop("dst_path", None)
     return (
         function_name,
         kwargs,
     )  # return name of mlflow function and kwargs for that function
 
 
-def request_log_model(function_name, kwargs, extra_attrs, module_name):
+def request_log_model(function_name: str, kwargs: dict, extra_attrs: list, module_name: str):
     """
     Send request for log_model function.
 
@@ -54,16 +56,9 @@ def request_log_model(function_name, kwargs, extra_attrs, module_name):
     3) Pack it to zip file
     4) Send it to server to log model there.
     """
-    extra_imports_args = [
-        "submodules",
-        "module_name",
-        "used_modules_names",
-        "extra_modules_names",
-        "root_module_name",
-        "linter_check",
-    ]
     delete_args_for_save_model_func = [
         "description",
+        "model_version_tags",
         "artifact_path",
         "registered_model_name",
         "await_registration_for",
@@ -76,7 +71,18 @@ def request_log_model(function_name, kwargs, extra_attrs, module_name):
         "source_executor_version",
         "source_executor_role",
         "start_build",
+        "create_venv_pack",
     ]  # not need for save_model
+
+    extra_imports_args = [
+        "submodules",
+        "module_name",
+        "used_modules_names",
+        "extra_modules_names",
+        "root_module_name",
+        "linter_check",
+    ]
+
     delete_args_for_log_func = [
         "python_model",
         "artifacts",
@@ -84,22 +90,23 @@ def request_log_model(function_name, kwargs, extra_attrs, module_name):
         "pip_requirements",
         "extra_pip_requirements",
     ]  # not need for log model on server
+
     for delete_arg in extra_imports_args:
-        del kwargs[delete_arg]
+        kwargs.pop(delete_arg, None)
     kwargs_for_save_model = kwargs.copy()
     for delete_arg in delete_args_for_save_model_func:
-        del kwargs_for_save_model[delete_arg]
+        kwargs_for_save_model.pop(delete_arg, None)
     python_model = kwargs_for_save_model["python_model"]
 
     # import some modules here because of circular import
-    from ML_management.dataset_loader_template.dataset_loader_pattern import DatasetLoaderPattern
-    from ML_management.dataset_loader_template.dataset_loader_pattern_to_methods_map import (
+    from ML_management.dataset_loader.dataset_loader_pattern import DatasetLoaderPattern
+    from ML_management.dataset_loader.dataset_loader_pattern_to_methods_map import (
         dataset_loader_pattern_to_methods,
     )
-    from ML_management.executor_template.executor_pattern_to_methods_map import executor_pattern_to_methods
-    from ML_management.executor_template.multi_model_executor_pattern import MultiModelExecutorPattern
-    from ML_management.models.model_type_to_methods_map import model_pattern_to_methods
-    from ML_management.models.patterns.model_pattern import Model
+    from ML_management.executor.base_executor import BaseExecutor
+    from ML_management.executor.executor_pattern_to_methods_map import executor_pattern_to_methods
+    from ML_management.model.model_type_to_methods_map import model_pattern_to_methods
+    from ML_management.model.patterns.model_pattern import Model
 
     with TemporaryDirectory() as temp_dir:
         model_folder = "model"
@@ -107,7 +114,7 @@ def request_log_model(function_name, kwargs, extra_attrs, module_name):
         zip_file_folder = "zip_file"
         path_for_zip_file = os.path.join(temp_dir, zip_file_folder)
         if python_model is not None:
-            if not isinstance(python_model, MultiModelExecutorPattern):
+            if not isinstance(python_model, BaseExecutor):
                 del kwargs["visibility"]
             if isinstance(python_model, Model):
                 kwargs["model_type"] = ModelType.MODEL
@@ -117,7 +124,7 @@ def request_log_model(function_name, kwargs, extra_attrs, module_name):
                     EXPERIMENT_NAME_FOR_DATASET_LOADER,
                 ]:
                     raise InvalidExperimentNameError(ModelType.MODEL.value, variables.active_experiment_name)
-            elif isinstance(python_model, MultiModelExecutorPattern):
+            elif isinstance(python_model, BaseExecutor):
                 kwargs["model_type"] = ModelType.EXECUTOR
                 model_to_methods = executor_pattern_to_methods
 
@@ -144,19 +151,29 @@ def request_log_model(function_name, kwargs, extra_attrs, module_name):
                         model_method = getattr(python_model, method_name_to_schema.name, None)
                         model_method_schema = infer_jsonschema(model_method)
                         methods_schema[method_name_to_schema.value] = model_method_schema
-            kwargs["model_method_schemas"] = methods_schema
-
-            mlflow.pyfunc.save_model(path=path_for_model_folder, **kwargs_for_save_model)
 
             for delete_arg in delete_args_for_log_func:
-                del kwargs[delete_arg]
-            kwargs["loader_module"] = mlflow.pyfunc.model.__name__
-            model_filename = shutil.make_archive(path_for_zip_file, "zip", path_for_model_folder)
+                kwargs.pop(delete_arg, None)
+            if function_name == "log_model":
+                kwargs["loader_module"] = mlflow.pyfunc.model.__name__
+                mlflow.pyfunc.save_model(path=path_for_model_folder, **kwargs_for_save_model)
+                model_filename = shutil.make_archive(path_for_zip_file, "zip", path_for_model_folder)
+            else:
+                artifacts_path = os.path.join(kwargs["model_path"], CONFIG_KEY_ARTIFACTS)
+                if os.path.isfile(artifacts_path):
+                    raise Exception(f"The artifact file {artifacts_path} is invalid. The artifact must be a directory.")
+
+                model_filename = shutil.make_archive(path_for_zip_file, "zip", kwargs["model_path"])
+                del kwargs["model_path"]
+                kwargs["loader_module"] = "ML_management.loader.loader"
+
+            kwargs["model_method_schemas"] = methods_schema
+
         else:
             raise Exception("python_model parameter must be specified")
 
         with open(model_filename, "rb") as file:
-            return request(function_name, kwargs, extra_attrs, file, module_name=module_name)
+            return request("log_model", kwargs, extra_attrs, file, module_name=module_name)
 
 
 def request_log_artifacts(function_name, kwargs, extra_attrs):
@@ -211,7 +228,7 @@ def send_request_to_server(function_name, kwargs, extra_attrs, for_class, module
     for example function log_model is in mlflow.pyfunc module (mlflow.pyfunc.log_model())
     """
     # special case for log_model, load_model, save_model
-    if function_name == "log_model":
+    if function_name == "log_model" or function_name == "log_object_src":
         response = request_log_model(function_name, kwargs, extra_attrs, module_name)
     elif function_name == "save_model":
         return mlflow.pyfunc.save_model(**kwargs)
@@ -248,6 +265,12 @@ def send_request_to_server(function_name, kwargs, extra_attrs, for_class, module
             }
             decoded_result = RestException(created_json)
         raise decoded_result
+    elif isinstance(decoded_result, AuthError) and (function_name == "log_model" or function_name == "log_object_src"):
+        decoded_result.args = (
+            f"{decoded_result.args[0]}. "
+            "Possible reason: you are trying to upload a version of an object owned by another user",
+        )
+        raise decoded_result
     elif isinstance(decoded_result, Exception):
         raise decoded_result
     return decoded_result
@@ -259,8 +282,8 @@ def _check_if_call_from_predict_function():
 
     Calls from predict function are prohibited and will do and return nothing.
     """
-    from ML_management.models.model_type_to_methods_map import ModelMethodName
-    from ML_management.models.patterns.model_pattern import Model
+    from ML_management.model.model_type_to_methods_map import ModelMethodName
+    from ML_management.model.patterns.model_pattern import Model
 
     predict_func_name = ModelMethodName.predict_function.name
 
