@@ -19,6 +19,10 @@ print_executor_schema = sdk.print_executor_schema
 print_executor_roles = sdk.print_executor_roles
 get_logs = sdk.get_logs
 get_build_logs = sdk.get_build_logs
+get_venv_build_logs = sdk.get_venv_build_logs
+serve_model = sdk.serve_model
+stop_model_serving = sdk.stop_model_serving
+check_inference_model_readiness = sdk.check_inference_model_readiness
 get_required_classes_by_executor = sdk.get_required_classes_by_executor
 available_metrics = sdk.available_metrics
 metric_history = sdk.metric_history
