@@ -1,18 +1,23 @@
 """SDK for client library."""
 import json
+import locale
 import posixpath
 import sys
+from enum import Enum
 from typing import Any, Dict, List, Optional, Union
+from urllib.parse import quote
 
 import matplotlib.pyplot as plt
 import pandas as pd
+import requests
+import websocket
 from jsf import JSF
 from sgqlc.operation import Operation
 
-from ML_management.executor_template.executor_pattern import JobExecutorPattern
+from ML_management.executor.base_executor import BaseExecutor
 from ML_management.mlmanagement import get_server_url, get_server_websocket_url
 from ML_management.mlmanagement.session import AuthSession
-from ML_management.models.model_type_to_methods_map import ModelMethodName
+from ML_management.model.model_type_to_methods_map import ModelMethodName
 from ML_management.registry.registry_manager import RegistryManager
 from ML_management.sdk import schema
 from ML_management.sdk.schema import ExecutionJob, ModelVersion
@@ -29,6 +34,14 @@ MODEL_INPUT_EXAMPLE = """{
 }"""
 
 
+class JobType(str, Enum):
+    """Enum job type."""
+
+    build = "build"
+    execution = "execution"
+    venv = "venv"
+
+
 def _to_datetime(df: pd.DataFrame, column_names: List[str]) -> pd.DataFrame:
     """
     Convert df's columns to datetime.
@@ -54,18 +67,34 @@ def _to_datetime(df: pd.DataFrame, column_names: List[str]) -> pd.DataFrame:
 def send_graphql_request(op: Operation, json_response: bool = True) -> Any:
     """Send request to server and process the response."""
     json_data = AuthSession().sgqlc_request(op)
+
     if "data" not in json_data or json_data["data"] is None:
+        server_url = get_server_url()
+        try:
+            if locale.getdefaultlocale()[0][:2] == "ru":
+                url_base = posixpath.join(server_url, "locales/ru/ru.json")
+            else:
+                url_base = posixpath.join(server_url, "locales/en/en.json")
+        except Exception:
+            # if there is no locale file use english by default.
+            url_base = posixpath.join(server_url, "locales/en/en.json")
+        translation = requests.get(url_base).json()
+
         error_message = json_data["errors"][0]["message"]
+        message_type, message_value = error_message.split(".")
+        if message_type not in translation:
+            raise Exception(message_type)
+
+        formatted_translated_message = translation[message_type][message_value]
         if (
             ("extensions" in json_data["errors"][0])
             and ("params" in json_data["errors"][0]["extensions"])
             and (json_data["errors"][0]["extensions"]["params"] is not None)
-            and ("message" in json_data["errors"][0]["extensions"]["params"])
         ):
-            error_extensions_message = json_data["errors"][0]["extensions"]["params"]["message"]
-            raise Exception(f"{error_message}\nMessage: {error_extensions_message}")
-        else:
-            raise Exception(error_message)
+            raise Exception(
+                formatted_translated_message.format().format(**json_data["errors"][0]["extensions"]["params"])
+            )
+        raise Exception(formatted_translated_message)
 
     if json_response:
         return json_data["data"]
@@ -206,6 +235,8 @@ def add_ml_job_single_model(
     dataset_loader_version: Optional[int] = None,
     model_version: Optional[int] = None,
     new_model_name: Optional[str] = None,
+    prepare_new_model_inference: bool = False,
+    new_model_description: Optional[str] = None,
     experiment_name: str = "Default",
     collector_name: str = "s3",
     choice_criteria: str = "latest",
@@ -213,6 +244,7 @@ def add_ml_job_single_model(
     periodic_type: str = "ONCE",
     metric_name: Optional[str] = None,
     optimal_min: bool = False,
+    additional_system_packages: Optional[List[str]] = None,
 ) -> str:
     """
     Create execution job.
@@ -251,6 +283,8 @@ def add_ml_job_single_model(
     new_model_name: Optional[str] = None
         Name of the model to save in case new model is to be created as a result of job execution
         (regulated by executor.upload_model_mode). Default: None.
+    new_model_description: Optional[str] = None
+        Description of the new model name. Default: None.
     experiment_name: str = "Default"
         Name of the experiment. Default: "Default"
     collector_name: {"s3", }
@@ -266,7 +300,8 @@ def add_ml_job_single_model(
         Name of the metric(must be logged in) for choice_criteria. Default: None
     optimal_min: bool = False
         Whether to take minimum or maximum value for the selected choice_criteria and metric_name. Default: False
-
+    additional_system_packages: Optional[List[str]] = None
+        List of system libraries for Debian family distributions that need to be installed in the job. Default: None
 
     Returns
     -------
@@ -275,7 +310,7 @@ def add_ml_job_single_model(
     """
     models = [
         {
-            "role": JobExecutorPattern.DEFAULT_ROLE,
+            "role": BaseExecutor.DEFAULT_ROLE,
             "model": {
                 "name": model_name,
                 "version": model_version,
@@ -285,6 +320,8 @@ def add_ml_job_single_model(
             },
             "params": model_params,
             "new_model_name": new_model_name,
+            "prepare_new_model_inference": prepare_new_model_inference,
+            "description": new_model_description,
         }
     ]
 
@@ -302,6 +339,7 @@ def add_ml_job_single_model(
         collector_name=collector_name,
         cron_expression=cron_expression,
         periodic_type=periodic_type,
+        additional_system_packages=additional_system_packages,
     )
 
 
@@ -319,6 +357,7 @@ def add_ml_job(
     collector_name: str = "s3",
     cron_expression: Optional[str] = None,
     periodic_type: str = "ONCE",
+    additional_system_packages: Optional[List[str]] = None,
 ) -> str:
     """
     Create execution job.
@@ -349,6 +388,7 @@ def add_ml_job(
                     "params": [{"method_one": {key1: value1, key2: value2}},
                                {"method_two": {key1: value1, key2: value2}}],
                     "new_model_name": "some_name"  # Optional. Default: None
+                    "description": "some_description" # Optional. Default: None
                 },
                 {
                     "role": "role_two",
@@ -388,6 +428,8 @@ def add_ml_job(
         Name of the metric(must be logged in) for choice_criteria. Default: None
     optimal_min: bool = False
         Whether to take minimum or maximum value for the selected choice_criteria and metric_name. Default: False
+    additional_system_packages: Optional[List[str]] = None
+        List of system libraries for Debian family distributions that need to be installed in the job. Default: None
 
     Returns
     -------
@@ -500,11 +542,15 @@ def add_ml_job(
                 )
 
         new_model_name = model.get("new_model_name")
+        new_model_description = model.get("description")
+        prepare_new_model_inference = model.get("prepare_new_model_inference", False)
 
         current_model_params = schema.ModelParamsInput(
             model_version_choice=model_version_choice,
             list_model_method_params=model_methods_params,
             new_model_name=new_model_name,
+            prepare_new_model_inference=prepare_new_model_inference,
+            description=new_model_description,
         )
 
         model_role = schema.RoleModelParamsInput(role=model["role"], model_params=current_model_params)
@@ -526,6 +572,7 @@ def add_ml_job(
             cron_expression=cron_expression,
             periodic_type=periodic_type,
             gpu=gpu,
+            additional_system_packages=additional_system_packages,
         )
     )
 
@@ -698,7 +745,7 @@ def model_version_metainfo(model_name: str, model_version: Optional[int] = None)
     base_query.version()
     base_query.status()
     base_query.build_job().status()
-    base_query.build_job().docker_image_tag()
+    base_query.build_job().build_object_name()
     base_query.available_executor_versions.name()
     base_query.available_executor_versions.version()
     model_version = send_graphql_request(op, json_response=False)
@@ -766,7 +813,7 @@ def _get_model_schema_for_executor(executor_name: str, executor_version: Optiona
     for model in models:
         model_name = model["name"]
         model_version = model.get("version") or int(RegistryManager().get_latest_model_version(name=model_name).version)
-        model_role = model.get("role", JobExecutorPattern.DEFAULT_ROLE)
+        model_role = model.get("role", BaseExecutor.DEFAULT_ROLE)
 
         current_model = schema.ObjectVersionInput(name=model_name, version=model_version)
         current_role = schema.RoleModelVersionInput(role=model_role, model=current_model)
@@ -986,16 +1033,16 @@ def print_executor_roles(name: str, version: Optional[int] = None) -> None:
     print(json_data["executorVersionFromNameVersion"]["desiredModelMethods"])
 
 
-def _get_logs_url(job_identity: str, is_execution: bool = True, stream: bool = True) -> str:
+def _get_logs_url(params: Dict[str, Any], job_type: JobType, stream: bool = True) -> str:
     server_url = get_server_websocket_url() if stream else get_server_url()
     url_base = posixpath.join(server_url, "logs-api")
-    job_type = "execution" if is_execution else "build"
-    local_path = f"{f'stream/{job_type}' if stream else f'filedump/{job_type}'}?identity={job_identity}"
+    url_encoded_params = "&".join(f"{quote(str(key))}={quote(str(value))}" for key, value in params.items())
+    local_path = f"{f'stream/{job_type.value}' if stream else f'filedump/{job_type.value}'}?{url_encoded_params}"
     return posixpath.join(url_base, local_path)
 
 
-def _get_logs(job_identity: str, is_execution: bool, stream: bool = True, file_name: Optional[str] = None) -> None:
-    url = _get_logs_url(job_identity=job_identity, stream=stream, is_execution=is_execution)
+def _get_logs(job_type: JobType, params: Dict[str, Any], stream: bool = True, file_name: Optional[str] = None) -> None:
+    url = _get_logs_url(params=params, stream=stream, job_type=job_type)
     if stream:
         ws = None
         file = sys.stdout
@@ -1004,12 +1051,26 @@ def _get_logs(job_identity: str, is_execution: bool, stream: bool = True, file_n
             if file_name:
                 file = open(file_name, "w")
             while True:
-                data = json.loads(ws.recv())
+                data = ws.recv()
+                if not data:
+                    return
+                data = json.loads(data)
                 if "status" not in data or data["status"] != "OK":
                     raise RuntimeError("Internal Server Error")
                 print("\n".join(data["logs"]), file=file)
         except KeyboardInterrupt:
             pass
+        except websocket._exceptions.WebSocketBadStatusException:
+            possible_reasons_msg = (
+                "Model with that name and version does not exist or does not have build job"
+                if job_type is not JobType.execution
+                else "Job with that name does not exist"
+            )
+            print(
+                "<Connection refused. Check your query parameters, they may be incorrect.>\n"
+                "<Possible reasons:>\n"
+                f"<{possible_reasons_msg}>"
+            )
         except Exception as err:
             print(err)
         finally:
@@ -1038,28 +1099,144 @@ def get_logs(job_name: str, stream: bool = True, file_name: Optional[str] = None
     ----------
     job_name: str
         Name of the execution job whose logs we want to view.
-    stream: bool
+    stream: bool = True
         Stream logs or dump all available at the moment.
     file_name: Optional[str] = None
         Name of the file where to save logs. Default: None. If None prints logs to the output.
     """
-    _get_logs(is_execution=True, job_identity=job_name, stream=stream, file_name=file_name)
+    _get_logs(job_type=JobType.execution, stream=stream, file_name=file_name, params={"job_id": job_name})
 
 
-def get_build_logs(docker_image_tag: str, stream: bool = True, file_name: Optional[str] = None) -> None:
+def get_build_logs(
+    model_name: str, model_version: Optional[int] = None, stream: bool = True, file_name: Optional[str] = None
+) -> None:
     """
-    Stream logs of the build job by docker image tag.
+    Stream logs of the build job by model name and version.
 
     Parameters
     ----------
-    docker_image_tag: str
-        The tag of the docker image whose creation logs you want to view.
-    stream: bool
+    model_name: str
+        The name of the model whose docker image creation logs you want to view.
+    model_version: Optional[int] = None
+        The version of the model whose docker image creation logs you want to view.
+        Default: None, "latest" version is used.
+    stream: bool = True
         Stream logs or dump all available at the moment.
     file_name: Optional[str] = None
         Name of the file where to save logs. Default: None. If None prints logs to the output.
     """
-    _get_logs(is_execution=False, job_identity=docker_image_tag, stream=stream, file_name=file_name)
+    model_version = (
+        int(RegistryManager().get_latest_model_version(name=model_name).version)
+        if model_version is None
+        else model_version
+    )
+    _get_logs(
+        job_type=JobType.build,
+        stream=stream,
+        file_name=file_name,
+        params={"model_name": model_name, "model_version": model_version},
+    )
+
+
+def get_venv_build_logs(
+    model_name: str, model_version: Optional[int] = None, stream: bool = True, file_name: Optional[str] = None
+) -> None:
+    """
+    Stream logs of the venv archive creating job by model name and version.
+
+    Parameters
+    ----------
+    model_name: str
+        The name of the model whose venv archive creation logs you want to view.
+    model_version: Optional[int] = None
+        The version of the model whose venv archive creation logs you want to view.
+        Default: None, "latest" version is used.
+    stream: bool = True
+        Stream logs or dump all available at the moment.
+    file_name: Optional[str] = None
+        Name of the file where to save logs. Default: None. If None prints logs to the output.
+    """
+    model_version = (
+        int(RegistryManager().get_latest_model_version(name=model_name).version)
+        if model_version is None
+        else model_version
+    )
+    _get_logs(
+        job_type=JobType.venv,
+        stream=stream,
+        file_name=file_name,
+        params={"model_name": model_name, "model_version": model_version},
+    )
+
+
+def serve_model(name: str, version: Optional[int] = None, gpu: bool = False) -> str:
+    """
+    Start model serving in triton service.
+
+    Parameters
+    ----------
+    name: str
+        Name of the model.
+    version: Optional[int] = None
+        Version of the model. Default: None, "latest" version is used.
+    gpu: bool
+        Determine which device will be used: GPU or CPU. Default: False, CPU is used.
+
+    Returns
+    -------
+    str:
+        endpoint path to make inference requests for model version.
+    """
+    version = int(RegistryManager().get_latest_model_version(name=name).version) if version is None else version
+    op = Operation(schema.Mutation)
+    serving_parameters = schema.ModelServingInput(
+        model_version=schema.ObjectVersionInput(name=name, version=version), gpu=gpu
+    )
+    op.serve_model(serving_parameters=serving_parameters)
+    send_graphql_request(op)
+    return f"/v2/models/{name}/versions/{version}/infer"
+
+
+def stop_model_serving(name: str, version: Optional[int] = None) -> None:
+    """
+    Stop model serving in triton service.
+
+    Parameters
+    ----------
+    name: str
+        Name of the model.
+    version: Optional[int] = None
+        Version of the model. Default: None, "latest" version is used.
+    """
+    version = int(RegistryManager().get_latest_model_version(name=name).version) if version is None else version
+    op = Operation(schema.Mutation)
+    op.stop_model_serving(model_version=schema.ObjectVersionInput(name=name, version=version))
+    send_graphql_request(op=op)
+    print("Serving has been successfully stopped.")
+
+
+def check_inference_model_readiness(name: str, version: Optional[int] = None) -> Optional[str]:
+    """
+    Check is model ready to accept requests.
+
+    Parameters
+    ----------
+    name: str
+        Name of the model.
+    version: Optional[int] = None
+        Version of the model. Default: None, "latest" version is used.
+
+    Returns
+    -------
+    Optional[str]:
+        endpoint path to make requests if model is ready else None
+    """
+    version = int(RegistryManager().get_latest_model_version(name=name).version) if version is None else version
+    op = Operation(schema.Query)
+    op.is_inference_model_ready(model_version=schema.ObjectVersionInput(name=name, version=version))
+    json_data = send_graphql_request(op=op)
+    result = f"/v2/models/{name}/versions/{version}/infer" if json_data["isInferenceModelReady"] else None
+    return result
 
 
 def get_required_classes_by_executor(name: str, version: Optional[int] = None) -> List[str]:
